// Toaster control state machine
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

#include "globals.h"
#include "abimport.h"
#include "proto.h"

struct action {
		void (*t_fun)( int );	// Transition routine
		int	next_state;			// Next state
		} action[ NUM_STATES ][ NUM_EVENTS ] = {

// S_MENU					transition      Next state
//                          routine
	/* E_OVERTEMP		*/	&t_noop,		S_MENU,
	/* E_TOAST			*/	&t_cook,		S_TOASTING,
	/* E_EJECT			*/	&t_noop,		S_MENU,
	/* E_SLIDER			*/	&t_select,		S_MENU,
	/* E_TIMER			*/	&t_noop,		S_MENU,
                                            
// S_TOASTING				transition      Next state
//                          routine
	/* E_OVERTEMP		*/	&t_eject,		S_MENU,
	/* E_TOAST			*/	&t_noop,		S_TOASTING,
	/* E_EJECT			*/	&t_eject,		S_MENU,
	/* E_SLIDER			*/	&t_adjust,		S_TOASTING,
	/* E_TIMER			*/	&t_animate,		S_TOASTING,

	};

// Put toaster into power-up condition
void
init() {

	PtSetParentWidget( ABW_base );
	curr_toast = ApCreateWidget( toaster_db, "toast1", 277, 350, 0, NULL );
	curr_shade = 1;
	state = S_MENU;

	timer_on( 1 );

	}

//-- Miscellaneous support routines

void
element_on() {

	PtArg_t		args[1];

	// Turn the heating element on
	PtSetArg( &args[0], Pt_ARG_FILL_COLOR, Pg_RED, 0 );
	PtSetResources( ABW_element, 1, args );
	
	}

void
element_off() {

	PtArg_t		args[1];

	// Turn the heating element off
	PtSetArg( &args[0], Pt_ARG_FILL_COLOR, Pg_DGREY, 0 );
	PtSetResources( ABW_element, 1, args );
	
	}

void
eject_toast() {
	PtDestroyWidget( big_toast );
	PtDestroyWidget( inside );
	big_toast = NULL;
	}

//-- Transition routines -----------------------------------

void
t_noop( int value ) {
	value = value;
	}

// Routine to animate toast button while toasting
void
t_animate( int value ) {

	int		shade = 1;
	char			buffer[20];
	PtLabelWidget_t	*limage;
	PhImage_t		*img;

	req_timer += ( value * 10 );

	if ( req_timer > 60 )	// 1:00 minute
		shade = 2;
	if ( req_timer > 120 )	// 2:00 minutes
		shade = 3;
	if ( req_timer > 150 )	// 2:30 minutes
		shade = 4;
	if ( req_timer > 180 )	// 3:00 minutes
		shade = 5;
	if ( req_timer > 210 )	// 3:30 minutes
		shade = 6;
	if ( req_timer > 240 )	// 4:00 minutes
		shade = 7;
	if ( req_timer > 270 )	// 4:30 minutes
		shade = 8;
	if ( req_timer > 300 )	// 5:00 minutes
		shade = 9;
	if ( req_timer > 330 )	// 5:30 minutes
		shade = 10;
	if ( req_timer > 360 )	// 6:00 minutes
		shade = 11;

	if ( shade != big_shade ) {
		sprintf( buffer, "btoast%d", shade );
		img = ApGetImageRes( toaster_db, buffer );
		big_toast->flags |= Pt_OPAQUE;
		limage = (PtLabelWidget_t *) big_toast;
		memcpy( limage->data, img, sizeof( PhImage_t ) );
		PtDamageWidget( big_toast );
		big_shade = shade;
		if ( big_shade >= req_shade )
			process_event( E_EJECT, 0 );
		}
	
	}

void
t_cook( int value ) {

	PtSetParentWidget( ABW_base );
	PtRealizeWidget( inside = ApCreateWidget( toaster_db, "inside", 203, 36, 0, NULL ) );
	PtSetParentWidget( ABW_base );
	PtRealizeWidget( big_toast = ApCreateWidget( toaster_db, "btoast1", 240, 59, 0, NULL ) );
	big_shade = value;
	req_shade = curr_shade;
	req_timer = 0;

	element_on();

	}

void
t_eject( int value ) {

	PtArg_t			args[1];

	value = value;
	element_off();
	eject_toast();

	PtSetArg( &args[0], Pt_ARG_GAUGE_VALUE, 0, 0 );
	PtSetResources( ABW_slider, 1, args );

	}

// Routine to select toast darkness
void
t_select( int value ) {

	char			buffer[20];
	PtLabelWidget_t	*limage;
	PhImage_t		*img;

	/* change toast darkness if new value */
	if ( curr_shade != value ) {
		/* remove flicker! */
		sprintf( buffer, "toast%d", value );
		img = ApGetImageRes( toaster_db, buffer );
		curr_toast->flags |= Pt_OPAQUE;
		limage = (PtLabelWidget_t *) curr_toast;
		memcpy( limage->data, img, sizeof( PhImage_t ) );
		PtDamageWidget( curr_toast );
		curr_shade = value;
		}

	}

// Routine to adjust desired toast darkness while toasting
void
t_adjust( int value ) {

	char			buffer[20];
	PtLabelWidget_t	*limage;
	PhImage_t		*img;

	/* change toast darkness if new value */
	if ( curr_shade != value ) {
		/* remove flicker! */
		sprintf( buffer, "toast%d", value );
		img = ApGetImageRes( toaster_db, buffer );
		curr_toast->flags |= Pt_OPAQUE;
		limage = (PtLabelWidget_t *) curr_toast;
		memcpy( limage->data, img, sizeof( PhImage_t ) );
		PtDamageWidget( curr_toast );
		curr_shade = value;
		if ( curr_shade <= big_shade )
			process_event( E_EJECT, 0 );
		else
			req_shade = curr_shade;
		}

	}

//-- End of transition routines ------------------------------

void
process_event( int event, int value ) {

	// Call the transition routine at this state indicated by the passed event
	action[ state ][ event ].t_fun( value );
	state = action[ state ][ event ].next_state;

	}

